package com.edgec.browserbackend.browser.service.Impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.edgec.browserbackend.account.domain.Account;
import com.edgec.browserbackend.account.exception.AccountErrorCode;
import com.edgec.browserbackend.account.repository.AccountRepository;
import com.edgec.browserbackend.browser.ErrorCode.BrowserErrorCode;
import com.edgec.browserbackend.browser.domain.IpResource;
import com.edgec.browserbackend.browser.domain.IpStatus;
import com.edgec.browserbackend.browser.domain.IpType;
import com.edgec.browserbackend.browser.dto.*;
import com.edgec.browserbackend.browser.repository.IpResourceRepository;
import com.edgec.browserbackend.browser.service.IpResourceService;
import com.edgec.browserbackend.common.commons.error.ClientRequestException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.time.Instant;
import java.util.*;

@Service
public class IpResourceServiceImpl implements IpResourceService {

    private final Logger logger = LoggerFactory.getLogger(IpResourceServiceImpl.class);

    private static String CLOUDAMURL = "https://www.cloudam.cn";
    private static String TESTURL = "http://112.74.13.2";
    private static String USERNAME = "fangguanlianbrowser";

    private static List<Interval> port = Arrays.asList(new Interval("22"), new Interval("443"), new Interval("20000"), new Interval("20001"));

    private static List<String> protocol = Arrays.asList("http", "https", "socks5", "ssr");

    @Autowired
    private AccountRepository accountRepository;

    @Autowired
    private IpResourceRepository ipResourceRepository;

    public HttpHeaders buildPostHeader() {
        HttpHeaders header = new HttpHeaders();
        header.setContentType(MediaType.APPLICATION_JSON);
        header.setBearerAuth("oq5tg3gMsflHcK5iZ2741G5R30XYd9blyOqH9qeBItKtrzfTsGIoy8AsxqqNXdcm");
        return header;
    }

    public HttpHeaders buildGetHeader() {
        HttpHeaders headers = new HttpHeaders();
        headers.setBearerAuth("oq5tg3gMsflHcK5iZ2741G5R30XYd9blyOqH9qeBItKtrzfTsGIoy8AsxqqNXdcm");
        return headers;
    }

    public static String makeRandomPassword(int len){
        char charr[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890~!@#$%^&*.?".toCharArray();
        StringBuilder sb = new StringBuilder();
        Random r = new Random();
        for (int x = 0; x < len; ++x) {
            sb.append(charr[r.nextInt(charr.length)]);
        }
        return sb.toString();
    }

    @Override
    public List<IpResourceDto> buyIp(String username, IpResourceRequestDto ipResourceRequestDto) {
        Account account = accountRepository.findByName(username);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        List<IpResourceDto> ipResourceDtos = new ArrayList<>();
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders header = buildPostHeader();
        HashMap<String, Object> map = new HashMap<>();
        map.put("name", ipResourceRequestDto.getName());
        map.put("region", ipResourceRequestDto.getRegion());
        map.put("period", String.valueOf(ipResourceRequestDto.getPeriod()));
        map.put("provider", ipResourceRequestDto.getVendor());
        map.put("unit", ipResourceRequestDto.getUnit());
        map.put("amount", String.valueOf(ipResourceRequestDto.getAmount()));
        map.put("loginPassword", makeRandomPassword(16));
        HttpEntity<Map<String, Object>> httpEntity = new HttpEntity<>(map, header);
        IpBuyResultDto ipBuyResultDto = null;
        try {
            ipBuyResultDto = restTemplate.postForObject(TESTURL + "/intelligroup/ipresources?accountId=browser", httpEntity, IpBuyResultDto.class);
            if (StringUtils.isNotBlank(ipBuyResultDto.getErrorCode()))
                throw new Exception(ipBuyResultDto.getErrorCode());
        } catch (Exception e) {
            logger.error("fail to post request", e.getMessage());
            logger.error(e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.INFORMATIONNOTCOMPELETE);
        }
        try {
            if (ipBuyResultDto != null && ipBuyResultDto.getIplist() != null && ipBuyResultDto.getIplist().size() < 1) {
                ipBuyResultDto.getIplist().forEach(x -> {
//                    IpInfoResultDto ipInfoResultDto = new IpInfoResultDto();
//                    Map<String, String> params = new HashMap<String, String>();
//                    params.put("accountId", "browser");
//                    params.put("ip", x.getIp());
//                    HttpHeaders headers = buildGetHeader();
//                    HttpEntity<Map<String, String>> entity = new HttpEntity<>(params, headers);
//                    ResponseEntity<String> result = restTemplate.exchange(TESTURL + "/ecc/ipinfo?accountId={accountId}&ip={ip}", HttpMethod.GET, entity, String.class);
//                    JSONObject jsonObject = JSON.parseObject(result.getBody());
//                    if (ipInfoResultDto != null && StringUtils.isBlank(ipInfoResultDto.getErrorCode())) {
                        IpResource ipResource = new IpResource();
                        ipResource.setIpAddr(x.getIp());
                        ipResource.setIpType(IpType.VENDOR);
//                        ipResource.setPurchasedTime(Long.valueOf((String) jsonObject.get("createdWhen")));
//                        ipResource.setValidTime(Long.valueOf((String)jsonObject.get("validTill")));
                        ipResource.setPurchasedTime(Instant.now().toEpochMilli());
                        if (ipResourceRequestDto.getUnit().equals("month"))
                            ipResource.setValidTime(Instant.now().plusSeconds(60*60*24*30).toEpochMilli());
                        else if (ipResourceRequestDto.getUnit().equals("week"))
                            ipResource.setValidTime(Instant.now().plusSeconds(60*60*24*7).toEpochMilli());
                        ipResource.setPort(port);
                        ipResource.setVendor(ipResourceRequestDto.getVendor());
                        ipResource.setIpStatus(IpStatus.NORMAL);
                        ipResource.setUsername(username);
                        if (account.getParent() != null)
                        ipResource.setUserParent(account.getParent());
                        ipResource.setRegion(ipResourceRequestDto.getRegion());
                        ipResource.setProtocol(protocol);
                        ipResourceRepository.save(ipResource);
                        ipResourceDtos.add(new IpResourceDto(ipResource, null));
//                    }
                });
            }
        } catch (Exception e) {
            logger.error("fail to post request", e.getMessage());
            logger.error(e.getMessage());
            throw new ClientRequestException(BrowserErrorCode.UNKNOWN);
        }
        return ipResourceDtos;
    }

    @Override
    public void renewIp(String username, IpResourceRequestDto ipResourceRequestDto) throws Exception {
        Account account = accountRepository.findByName(username);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        RestTemplate restTemplate = new RestTemplate();
        HashMap<String, Object> map = new HashMap<>();
        map.put("iplist", ipResourceRequestDto.getIplist());
        map.put("period", ipResourceRequestDto.getPeriod());
        HttpHeaders headers = buildPostHeader();
        HttpEntity<Map<String, Object>> entity = new HttpEntity<>(map, headers);
        ipResourceRequestDto.getIplist().forEach(x -> {
            IpResource ipResource = ipResourceRepository.findByAddrAndIsDeleted(x, false);
            if (ipResource == null)
                throw new ClientRequestException(BrowserErrorCode.IPNOTEXIST);
        });
        try {
            ResponseEntity<String> result = restTemplate.exchange(TESTURL + "/intelligroup/renewip?accountId=browser", HttpMethod.PUT, entity, String.class);
            RenewIpResultDto renewIpResultDto = JSON.parseObject(result.getBody(), RenewIpResultDto.class);
            if (StringUtils.isNotBlank(renewIpResultDto.getErrorCode()))
                throw new Exception(renewIpResultDto.getErrorCode());
            renewIpResultDto.getIplist().forEach(x -> {
                IpResource ipResource = ipResourceRepository.findByAddrAndIsDeleted(x.getIp(), false);
                ipResource.setValidTime(Instant.parse(x.getValidTill()).toEpochMilli());
                ipResourceRepository.save(ipResource);
            });
        } catch (Exception e) {
            logger.error("fail to renew ip", e.getMessage());
            logger.error(e.getMessage());
            throw new Exception(e.getMessage());
        }
    }

    @Override
    public void deleteIp(String username, String ipAddr) throws Exception {
        Account account = accountRepository.findByName(username);
        if (account == null)
            throw new ClientRequestException(AccountErrorCode.NAMENOTEXIST);
        IpResource ipResource = ipResourceRepository.findByAddrAndIsDeleted(ipAddr, false);
        if (ipResource == null)
            throw new ClientRequestException(BrowserErrorCode.IPNOTEXIST);
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = buildGetHeader();
        Map<String, String> params = new HashMap<String, String>();
        params.put("id", "1");
        HttpEntity<Map<String, String>> httpEntity = new HttpEntity<>(params, headers);
        try {
            ResponseEntity<String> result = restTemplate.exchange(TESTURL + "/intelligroup/ipresources?accountId=browser&ip={ip}", HttpMethod.DELETE, httpEntity, String.class);
            DeleteIpResultDto deleteIpResultDto = JSON.parseObject(result.getBody(), DeleteIpResultDto.class);
            if (StringUtils.isNotBlank(deleteIpResultDto.getErrorCode()))
                throw new Exception(deleteIpResultDto.getErrorCode());
            ipResourceRepository.delete(ipResource);
        } catch (Exception e) {
            logger.error("fail to renew ip", e.getMessage());
            logger.error(e.getMessage());
            throw new Exception(e.getMessage());
        }
    }

    @Override
    public List<IpResourceDto> getIpList(String username, int type, int page, int amount, FilterDto filterDto) {
        return null;
    }


}
